#ifndef MOTION_HANDLER_H
#define MOTION_HANDLER_H

#include <boost/thread.hpp>

#include "Utils\ReferencePoint.h"
#include "Path_planning\Motion_sequence.h"
#include "Manager\Safe_type.h"
#include "Manager\Writen_paths_filenames.h"


template <typename Kernel>
class Motion_handler
{
public:
  typedef Reference_point<Kernel>     Ref_p;
  typedef Motion_sequence<Kernel>     Motion_sequence;
  typedef Motion_step_base<Kernel>    Motion_step;
  typedef Environment<Kernel>         Environment;
  
  typedef Safe_type<Ref_p>            Robot_location;
public:
  Motion_handler() {} //should be instansiated also with a 
                      //a pointer to the GUI
                      //a pointer to the planner
  void start( bool                    is_player_a,
              Environment*            env_ptr,
              Robot_location*         my_robot_ptr,
              Robot_location*         other_location_ptr,
              Writen_paths_filenames* writen_paths_filenames_ptr)
  {
    thread = boost::thread( &Motion_handler::handle_motion, this, 
                            env_ptr, my_robot_ptr, other_location_ptr, writen_paths_filenames_ptr);
    return;
  }
  void handle_motion( Environment*            env_ptr,            //needed for planner
                      Robot_location*         my_robot_ptr,       //needed for planner
                      Robot_location*         other_location_ptr, //needed for planner
                      Writen_paths_filenames* writen_paths_filenames_ptr)
  {
    std::vector<std::string> file_names_vec;
    writen_paths_filenames_ptr->pop_paths_filenames(std::back_inserter(file_names_vec));

    if (file_names_vec.empty())
      return;

    //read motion from motion file
    Motion_sequence motion_sequence;
    BOOST_FOREACH (std::string file_name, file_names_vec)
    {
      std::ifstream in(file_name.c_str());
      motion_sequence.read(in);
    }
    CGAL_precondition ( my_robot_ptr->get() == 
                        motion_sequence.get_sequence().front()->source() );

    //update location of robot
    Ref_p update_ref_p = motion_sequence.get_sequence().back()->target();
    my_robot_ptr->set(update_ref_p);

    //validate motion
    bool valid_motion = this->validate_motion_sequence(motion_sequence);
    CGAL_postcondition (valid_motion); //ToDo - change to show that path is wrong and end game

    //draw motion GUI
    this->draw_motion(motion_sequence);
  }
  bool validate_motion_sequence(Motion_sequence& motion_sequence)
  {
    for (unsigned int i(0); i < motion_sequence.get_sequence().size(); ++i)
    {
      Motion_step* motion_step_ptr = motion_sequence.get_sequence()[i];
      if (motion_step_ptr->type() == Motion_step::TRANSLATION)
      {
        if (validate_motion_step_translational(motion_step_ptr) == false)
          return false;
      }
      else // (motion_step.type() == Motion_step<Kernel>::ROTATION)
      {
        if (validate_motion_step_rotational(motion_step_ptr) == false)
          return false;
      }
    }
    return true;
  }
  bool validate_motion_step_translational(Motion_step* motion_step_ptr)
  {
    return true; //ToDo - validate...
  }
  bool validate_motion_step_rotational(Motion_step* motion_step_ptr)
  {
    return true; //ToDo - validate...
  }
  void draw_motion(Motion_sequence& motion_sequence)
  {
    return;
  }
  void join()
  {
    thread.join();
  }

private:
  boost::thread thread;
}; //Motion_handler
#endif //MOTION_HANDLER_H