#ifndef SYNCRONIZER_H
#define SYNCRONIZER_H

#include "Path_planning\Motion_sequence.h"
#include "Manager\Motion_handler.h"
#include "Manager\Safe_type.h"
#include "Time_frame.h"
#include <boost/thread.hpp>

template <typename Kernel>
class Syncronizer
{
public:
  typedef Reference_point<Kernel>     Ref_p;
  typedef Motion_sequence<Kernel>     Motion_sequence;
  typedef Environment<Kernel>         Environment;
  
  typedef Safe_type<Ref_p>            Robot_location;
  typedef Motion_handler<Kernel>      Motion_handler;

public:
  Syncronizer() {}
  void start( Time_frame*   time_frame,
              Environment*  env_ptr,
              Robot_location*         robot_a_location_ptr,
              Robot_location*         robot_b_location_ptr, 
              Writen_paths_filenames* writen_paths_filenames_a_ptr,
              Writen_paths_filenames* writen_paths_filenames_b_ptr)
  {
    thread = boost::thread( &Syncronizer::syncronize, this, 
                          time_frame, env_ptr,
                          robot_a_location_ptr, robot_b_location_ptr,
                          writen_paths_filenames_a_ptr, writen_paths_filenames_b_ptr);
    return;
  }
  void syncronize(Time_frame* time_frame, Environment*  env_ptr,
                  Robot_location* robot_a_location_ptr,
                  Robot_location* robot_b_location_ptr, 
                  Writen_paths_filenames* writen_paths_filenames_a_ptr,
                  Writen_paths_filenames* writen_paths_filenames_b_ptr)
  {
    time_frame->start();

    //first interval
    std::cout <<"in synchronizer thread : " <<boost::this_thread::get_id()<<std::endl;
    std::cout <<"before colored_sleep"<<std::endl;

    boost::posix_time::seconds colored_sleep_time(get_colored_sleep_time());
    boost::this_thread::sleep(colored_sleep_time);

    std::cout <<"in synchronizer thread : " <<boost::this_thread::get_id()<<std::endl;
    std::cout <<"before changing color"<<std::endl;
    time_frame->change_color(get_gray_sleep_time());
    
    //begining of gray interval - safety
    //wait untill interval is over and change color
    std::cout <<"in synchronizer thread : " <<boost::this_thread::get_id()<<std::endl;
    std::cout <<"before gray sleep"<<std::endl;
    boost::posix_time::seconds gray_sleep_time(get_gray_sleep_time());
    boost::this_thread::sleep(gray_sleep_time);

    std::cout <<"in synchronizer thread : " <<boost::this_thread::get_id()<<std::endl;
    std::cout <<"before changing color"<<std::endl;
    time_frame->change_color(get_colored_sleep_time());

    while(1)
    {
      //begining of colored interval
      //(1)   read paths that the player whose interval ended has writen
      //      validate them and update the GUI
      //(2)   wait untill interval is over and change color

      // (1)
      bool is_player_a (time_frame->get_color() == Time_frame::RED);
      Motion_handler motion_handler;
      if (is_player_a)
        motion_handler.start( true, env_ptr, 
                              robot_a_location_ptr, robot_b_location_ptr,
                              writen_paths_filenames_a_ptr);
      else //is_player_b
        motion_handler.start(false, env_ptr, 
                              robot_b_location_ptr, robot_a_location_ptr,
                              writen_paths_filenames_b_ptr);

      // (2)
      std::cout <<"in synchronizer thread : " <<boost::this_thread::get_id()<<std::endl;
      std::cout <<"before colored_sleep"<<std::endl;
      boost::posix_time::seconds colored_sleep_time(get_colored_sleep_time());
      boost::this_thread::sleep(colored_sleep_time);

      std::cout <<"in synchronizer thread : " <<boost::this_thread::get_id()<<std::endl;
      std::cout <<"before changing color"<<std::endl;
      time_frame->change_color(get_gray_sleep_time());
      
      //begining of gray interval - safety
      //wait untill interval is over and change color
      std::cout <<"in synchronizer thread : " <<boost::this_thread::get_id()<<std::endl;
      std::cout <<"before gray sleep"<<std::endl;
      boost::posix_time::seconds gray_sleep_time(get_gray_sleep_time());
      boost::this_thread::sleep(gray_sleep_time);

      std::cout <<"in synchronizer thread : " <<boost::this_thread::get_id()<<std::endl;
      std::cout <<"before changing color"<<std::endl;
      time_frame->change_color(get_colored_sleep_time());
    }
    return;
  }
  void join()
  {
    thread.join();
  }
public:
  double get_colored_sleep_time()
  {
    return 9;
  }
  double get_gray_sleep_time()
  {
    return 1;
  }
private:
  boost::thread thread;
}; //Syncronizer

#endif //SYNCRONIZER_H