#include "stdafx.h"
#include "Programs\client.h"

#include "Utils\Communication_utils\socket.h"
#include "Utils\Communication_utils\client_utils.h"
#include "Path_planning\Motion_sequence.h"

#include <boost/thread/thread.hpp>

typedef Motion_sequence<Rational_kernel> Motion;

Socket_client* socket_client_ptr = NULL;

void dbg_log(char* function_name, char* action)
{
  std::cout << "client, thread id " <<boost::this_thread::get_id()<<" " 
            << "in function " << function_name
            << " " <<action
            <<std::endl;
}


void plan(double remaining_time)
{
  dbg_log("plan", "entering");
  //here you should implement your planner
  //instead we just sleep for half of the remaining time
  boost::posix_time::seconds sleep_time(remaining_time);
  boost::this_thread::sleep(sleep_time);

  dbg_log("plan", "exiting");
  return;
}
void construct_motion(double remaining_time, Motion& motion_sequence)
{
  dbg_log("construct_motion", "entering");

  //construct a motion, 
  //make sure that you have enough time to complete the motion 
  //or else no write approval will granted
  //we just sleep this time ...
  boost::posix_time::seconds sleep_time(remaining_time);
  boost::this_thread::sleep(sleep_time);

  dbg_log("construct_motion", "exiting");
  return;
}

void move(double remaining_time)
{
  dbg_log("move", "entering");
  
  //here you should construct a motion
  Motion motion_sequence;
  construct_motion(remaining_time, motion_sequence);

  double motion_length(motion_sequence.motion_time( configuration.get_translational_speed(),
                                                    configuration.get_rotational_speed()));
  std::string path_filename;

  dbg_log("move", "requesting to write");
  if (request_to_write(*socket_client_ptr, motion_length, path_filename))
  {
    dbg_log("move", "request approved");
    //request approved
    ofstream out(path_filename.c_str(), std::ios::app); //it is essential to append
    motion_sequence.write(out);    
  }
  else
  {
    dbg_log("move", "request denied");
  }

  dbg_log("move", "exiting");
  return;
}
void moveable_planner(double remaining_time)
{
  dbg_log("moveable_planner", "entering");
  
  CGAL::Timer timer;
  timer.start();
  while (timer.time() < remaining_time)
  {
    //plan and then move (if time remains)
    plan(remaining_time - timer.time());
    if (remaining_time < timer.time())
      move(remaining_time - timer.time());
  }

  dbg_log("moveable_planner", "exiting");
  return;
}

void static_planner(double remaining_time)
{
  dbg_log("static_planner", "entering");

  CGAL::Timer timer;
  timer.start();
  while (timer.time() < remaining_time)
  {
    //plan
    plan(remaining_time - timer.time());
  }

  dbg_log("static_planner", "exiting");
  return;
}

void client_main(int argc, char* argv[])
{
  ////////////////////////////////////////////////////////////
  typedef Environment<> Env;
  Env env(argc,argv);
  Input_reader input_reader;
  

  ////////////////////////////////////////////////////////////
  //loading scene from environment
  //should be used in stub functions:
  //  plan, move, construct_motion
  Env::Polygon_vector&      workspace(env.get_workspace());
  Env::Extended_polygon&    robot(env.get_robot_a());
  Env::Extended_polygon&    dynamic_obstacle(env.get_robot_b());
  Env::Reference_point      source_configiration(env.get_source_configiration_a());
  Env::Reference_point      dynamic_obstacle_configiration(env.get_source_configiration_b());
  Env::Reference_point_vec& target_configurations(env.get_target_configirations());
    
  ////////////////////////////////////////////////////////////
  //initialize connection to server
  socket_client_ptr = new Socket_client(configuration.get_host_name(), configuration.get_host_port());
  //socket_client_ptr = new Socket_client("127.0.0.1", 1101);

  //while (1);

  ////////////////////////////////////////////////////////////
  //begin planning
  while (1)
  {    
    //before doing any planning, see if something changed
    dbg_log("main client", "getting scene status");
    Scene_status scene_status = get_scene_status(*socket_client_ptr); 
    dynamic_obstacle_configiration = 
      input_reader.read_reference_point<Rational_kernel>(scene_status.quasi_dynamic_obs_location_filename);
    if (scene_status.updated_target_configurations)
    {
      input_reader.read_reference_points<Rational_kernel>(scene_status.updated_target_configurations_filename,
                                                          std::back_inserter(target_configurations));
    }   
    
    //now plan
    dbg_log("main client", "planning");
    Time_frame_status time_frame_status = get_time_frame_status(*socket_client_ptr); 
    if (time_frame_status.is_moveable)
      moveable_planner(time_frame_status.remaining_time);
    else //(time_frame_status.is_moveable == false)
      static_planner(time_frame_status.remaining_time);
  }
}

