#include "stdafx.h"
#include "Programs\Server.h"
#include "Utils\Communication_utils\socket.h"
#include "Utils\Communication_utils\socket_typedefs.h"

#include "Manager\File_names.h"
#include "Manager\Syncronizer.h"
#include "Manager\Time_frame.h"
#include "Manager\Safe_type.h"
#include "Manager\Request_handler.h"
#include "Manager\Additional_target_configurations_manager.h"

#include <boost/thread/mutex.hpp>

void server_main(int argc, char* argv[])
{
  typedef Environment<>                     Env;
  typedef Safe_type<Env::Reference_point>   Robot_location;
  typedef Request_handler<Env::K>           Client_request_handler;


  ///////////////////////////////////////////////////////////////
  //set up global data structures
  Syncronizer<Env::K>                       syncronizer;

  Env                                       env(argc,argv);
  Time_frame                                time_frame( Time_frame::RED,
                                                        syncronizer.get_colored_sleep_time());
  File_names                                file_names(&time_frame);
  Additional_target_configurations_manager  additional_target_configurations_manager(5,&time_frame); //add additional targets at fifth interval
                                                                                    
  Robot_location                            robot_location_a(env.get_source_configiration_a());
  Robot_location                            robot_location_b(env.get_source_configiration_b());
  Writen_paths_filenames                    writen_paths_filenames_a;
  Writen_paths_filenames                    writen_paths_filenames_b;

  ///////////////////////////////////////////////////////////////
  //set up sockets 
  int host_port_a   = 1101;
  int host_port_b   = 1102;
  int conenctions = 1;

  std::cout <<"in main thread : " <<boost::this_thread::get_id()<<std::endl;
  std::cout <<"before connecting to client a"<<std::endl;
  
  Socket_server server_a(host_port_a, conenctions);
  //here program a should be executed
  Socket* socket_a=server_a.accept();
  Client_request_handler request_handler_a;
  request_handler_a.start(socket_a, Time_frame::RED,
                          &time_frame, &file_names,
                          &additional_target_configurations_manager,
                          &writen_paths_filenames_a,
                          &robot_location_b);

#if 0
  std::cout <<"in main thread : " <<boost::this_thread::get_id()<<std::endl;
  std::cout <<"before connecting to client b"<<std::endl;
  Socket_server server_b(host_port_b, conenctions);
  //here program a should be executed
  Socket* socket_b=server_b.accept();
  Client_request_handler request_handler_b;
  request_handler_b.start(socket_b, Time_frame::BLUE,
                          &time_frame, &file_names,
                          &additional_target_configurations_manager,
                          &writen_paths_filenames_b,
                          &robot_location_a);
#endif

  ///////////////////////////////////////////////////////////////
  //start up syncronizer
  std::cout <<"in main thread : " <<boost::this_thread::get_id()<<std::endl;
  std::cout <<"before synchronizer"<<std::endl;
  syncronizer.start(&time_frame, &env,
                    &robot_location_a, &robot_location_b,
                    &writen_paths_filenames_a, &writen_paths_filenames_b);
  
  syncronizer.join();
  request_handler_a.join();
//  request_handler_b.join();
  
  return ;
}
